	Function ELVGEO(flat,flong,glat,glong)
c     To compute the elevation angle of a geostationary satellite
c     whose sub-satellite point is glat,glong, as seen from an
c     arbitrary point on the earth.  The value of glat is typically
c     very close to zero.
c
	parameter(rgeo=42194., rgeo2=rgeo*rgeo)
*     Geostationary central distance in km
c
	arc = distan(flat,flong, glat,glong)/111.12  !!  arc length
	r = erad84(flat)  !!  radius of earth at flat
*     Use Law of Cosines:
	slant = sqrt(r*r + rgeo2 - 2.*r*rgeo*cosine(arc))
	sinb = rgeo*sine(arc)/slant
	elvgeo = 90. - arcsin(sinb)

	return
	end

c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Real Function DISTAN(alat,along, blat,blong)
c     To obtain the distance in km between two points on the
c     earth's surface: The arguments and result are REAL*4.
*     8 May 03
c
	Implicit Double Precision(d)
	Parameter(dr=3.1415926536d+0/180.d+0)
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	dalat = dr*alat
	dalong = dr*along
	dblat = dr*blat
	dblong = dr*blong
	dc = dsin(dalat)*dsin(dblat) +
     1  dcos(dalat)*dcos(dblat)*dcos(dalong-dblong)
c
	if(dabs(dc) .lt. 1.d+0) then
	dis = 111.12d+0 * dacos(dc)/dr
	  else
	dis = 0.d+0
	end if
c
	distan = dis
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function ERAD84(gdlat)
*     Earth radius in km as a function of geodetic latitude:
*
*     9 Jan 03; uses the spheroid WGS84
*     5 Jan 03; Liam Gumley reports that MODIS entails the WGS
c     spheroid of 1984, in which the Equatorial radius is
c     6378.137 km, and eccentricity .081819190843, which implies
c       a polar radius of 6356.75231424498 km.
*     b**2 = a**2(1 - e**2)
*
	Implicit Double Precision (d)
*
	Parameter (dmajor=6378.137d+0, dminor=6356.752314245d+0,
     1  b = dminor/dmajor, dmaj2=dmajor*dmajor,
     2  dratio=dminor/dmajor, degrad=3.1415926535898d+0/180.d+0)
*     This is the WGS84 Spheroid.
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	if(abs(gdlat) .gt. 90.) then
	write(*, '('' EARRAD: Given Latitude Is Out of Range: '',
     1  1pe16.4)') gdlat
	earrad = -1.
	end if
c
	gclat = geocen(gdlat)
*     converts to geocentric
	dxe = dmajor * dcos(degrad * gclat)
	dxe2 = dxe*dxe
	dye = dratio*dsqrt(dmaj2 - dxe2)
	erad84 = dsqrt(dxe2 + dye*dye)
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function SINE(xx)
*
c     To obtain the sine of an angle expressed in degrees.  Both
c     input and result are Real(*4).
c
	Implicit Double Precision(d)
	Parameter( dc = 180.d+0/3.141592653589793)
c     xxxxxxxxxxxxxxxxxxxxxxxxxx
	dxx = xx
	ds = dsin(dxx/dc)
	sine = ds
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function COSINE(xx)
c
c     To obtain the cosine of an angle expressed in degrees.  Both
c     input and result are Real(*4).
*
	Implicit Double Precision(d)
	Parameter( dc = 180.d+0/3.141592653589793)
c
	dxx = xx
	ds = dcos(dxx/dc)
	cosine = ds
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function ARCSIN(x)
*     Arcsin of an angle expressed in degrees
*
	Implicit None
	 Real x,xx,abs,arcsin,asin
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	xx = x
*
	if(abs(xx).gt.1.)then
	write(*, '('' Argument too large in ARCSIN: '', 1pe16.8)') xx
	call exit(99)
	end if
*
	arcsin = 57.2957795*asin(xx)
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function GEOCEN(gdl)
c
*     See GEODET for an explanation of this routine.
	parameter (smajx=6378388., sminx=6356911.946,
     1  b=sminx/smajx, b2 = b*b)
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	gd = abs(gdl)
*
	if(gd.ge.90.)then
	gc = 90.
*
	else
	tand = sine(gd)/cosine(gd)
	gc = arctan(1., b2*tand)
	end if
*
	geocen = sign(gc, gdl)
	return
	end
c     rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
	Function ARCTAN(xx,yy)
c
*     To compute arctangent in degrees, given x and y:
*     The output ranges from -180. to +180. deg.
*     The inputs and output are Real*4.
c
*     29 Aug 01; see Approximations for Digital Computers, Hastings.
*
	Implicit Double Precision(d)
	parameter(dpi=3.14159265358979d+0, d1=1.d+0,
     1  dpio4=dpi/4.d+0, d0=0.d+0, dr2d=180.d+0/dpi,
     1  dc1=.9999993329d+0, dc3=-.3332985605d+0,
     2  dc5=.1994653599d+0, dc7=-.1390853351d+0, dc9=.964200441d-1,
     3  dc11=-.559098861d-1, dc13=.218612288d-1, dc15=-.4054058d-2)
	save
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	dx = xx
	dy = yy
*
	if(dabs(dx).le.d0)then
	dd = 90.d+0
	if(dy.lt.d0)dd=-90.d+0
*
	else
	dq  = dabs(dy/dx)
	da = (dq - d1)/(dq + d1)
	da2 = da*da
	dr = dpio4 + da*(dc1 + da2*(dc3 + da2*(dc5 + da2*(dc7 + da2*(dc9
     1  + da2*(dc11 + da2*(dc13 + da2*dc15)))))))
	dd = dr2d*dr
*
	if(dx.gt.d0)then
	if(dy.lt.d0)dd=-dd
	 else
	 dd = 180.d+0 - dd
	if(dy.lt.d0)dd=-dd
	end if
	end if
*
	arctan = dd
	return
	end

