      REAL FUNCTION ESWAT( KEY, IUNIT, TEMP )

C----------------------------------------------------------------------
C !F77
C
C !DESCRIPTION:
C     DETERMINES SATURATION VAPOR PRESSURE WITH RESPECT TO WATER
C
C !INPUT PARAMETERS:
C     KEY = 1 FOR EXACT FORMULATION-INTEGRATION OF CLAUSIUS-CLAPEYRON
C             EQUATION BASED ON WORK BY GOFF-GRATCH
C         = 2 FOR TETENS BASE 10 FORMULATION ( HAURWITZ )
C         = 3 FOR TETENS BASE  E FORMULATION ( MURRAY )
C         = 4 FOR EXPONENTIAL-POLYNOMIAL FORMULATION ( RICHARDS-WIGLEY )
C         = 5 FOR POLYNOMIAL FORMULATION ( LOWE )
C     IUNIT = 1 FOR TEMPERATURE IN CENTIGRADE
C           = 2 FOR TEMPERATURE IN KELVIN
C     TEMP = TEMPERATURE
C
C !OUTPUT PARAMETERS:
C     ESWAT = SATURATION VAPOR PRESSURE OVER WATER ( MB )
C
C !REVISION HISTORY:
C
C !TEAM-UNIQUE HEADER:
C    Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C
C !DESIGN NOTES:
C     GOFF-GRATCH FORMULATION FROM SIXTH REVISED EDITION OF
C     SMITHSONIAN MET TBLS
C
C     TETENS, MURRAY, RICHARDS-WIGLEY, LOWE FORMULATIONS FROM
C     LOWE,JAM,16,1,1977
C
C !END
C-----------------------------------------------------------------------

c ... Scalar Arguments

      REAL TEMP
      INTEGER IUNIT,KEY

c ... Local Scalars

      REAL A,A0,A1,A2,A3,A4,A5,A6,B,B1,B2,B3,B4,C1,C2,C3,C4,C5,C6,
     +     CENKEL,CONH,CONM,CONRW,ES,EWS,T,TC,TN,TS,U,V

c ... Data statements

      DATA CENKEL/273.15/
      DATA TS,EWS/373.16,1013.246/
      DATA C1,C2,C3,C4,C5,C6/7.90298,5.02808,1.3816E-7,11.344,8.1328E-3,
     +     3.49149/
      DATA CONH,U,V/6.1078,7.5,237.3/
      DATA CONM,A,B/6.1078,17.2693882,35.86/
      DATA CONRW,B1,B2,B3,B4/1013.25,13.3185,-1.9760,-0.6445,-0.1299/
      DATA A0,A1,A2,A3,A4,A5,A6/6984.505294,-188.9039310,2.133357675,
     +     -1.288580973E-2,4.393587233E-5,-8.023923082E-8,
     +     6.136820929E-11/

      ESWAT = 0.0

      IF (KEY.LT.1 .OR. KEY.GT.5) RETURN
      IF (IUNIT.LT.1 .OR. IUNIT.GT.2) RETURN
      IF (IUNIT.EQ.1) T = TEMP + CENKEL
      IF (IUNIT.EQ.2) T = TEMP

      IF (KEY.EQ.1) THEN

C       GOFF-GRATCH

        ES = -C1* (TS/T-1.) + C2*ALOG10(TS/T) -
     +       C3* (10.** (C4* (1.-T/TS))-1.) +
     +       C5* (10.** (-C6* (TS/T-1.))-1.) + ALOG10(EWS)
        ESWAT = 10.0**ES
        IF (ESWAT.LT.0.0) ESWAT = 0.0

      ELSE IF (KEY.EQ.2) THEN

C       TETENS

        TC = T - CENKEL
        ESWAT = CONH*10.0** (U*TC/ (TC+V))
        IF (ESWAT.LT.0.0) ESWAT = 0.0

      ELSE IF (KEY.EQ.3) THEN

C       MURRAY

        ESWAT = CONM*EXP(A* (T-CENKEL)/ (T-B))
        IF (ESWAT.LT.0.0) ESWAT = 0.0

      ELSE IF (KEY.EQ.4) THEN

C       RICHARDS-WIGLEY

        TN = 1.0 - TS/T
        ESWAT = CONRW*EXP(TN* (B1+TN* (B2+TN* (B3+B4*TN))))
        IF (ESWAT.LT.0.0) ESWAT = 0.0

      ELSE IF (KEY.EQ.5) THEN


C       LOWE

        ESWAT = A0 + T* (A1+T* (A2+T* (A3+T* (A4+T* (A5+A6*T)))))
        IF (ESWAT.LT.0.0) ESWAT = 0.0
        IF (T.LT.200.0) ESWAT = 0.0

      ENDIF

      END

