c##########################################################################
      subroutine getwq0( cr, wq, tautr, tem, tau, nl, lsta, dbdt, tswq,
     $                  tsurfair, emis )

C=======================================================================
C
C                    getwq0
C
!F77====================================================================

!ROUTINE NAME: getwq0

!ABSTRACT:
c     this subroutine is for calculating the weighting function of
c     atmospheric absorbing constituent profile (h2o, o3, etc.)

!CALL PROTOCOL:

c     subroutine getwq0( cr,
c                       wq,
c                       tautr,
c                       tem,
c                       tau,
c                       nl,
c                       lsta,
c                       dbdt,
c                       tswq,
c                       tsurfair,
c                       emis )

!INPUT PARAMETERS:

C     type       name           purpose                            units
C     ------------------------------------------------------------------
c     real       cr             clear reflected radiance
c     real       tautr          tau(sfc)*tau(sfc)/tau(lev)
c     real       tem            temperature profile                k
c     real       tau            transmittance profile
c     integer    nl             level of atmosphere
c     integer    lsta           level of cloud top
c     real       dbdt           planck function derivative
c     real       tswq           retrieved surface temperature      k
c     real       tsurfair       clear air surface temperature      k
c     real       emis           emissivity

!OUTPUT PARAMETERS:

C     type       name           purpose            units
C     ------------------------------------------------------------------
c     real       wq             weighting functions

!INPUT/OUTPUT PARAMETERS:

C     type       name           purpose            units
C     ------------------------------------------------------------------

!RETURN VALUES:

!PARENT(S):

!ROUTINES CALLED:

C     name       purpose
C     ------------------------------------------------------------------

!FILES ACCESSED:

C     name       description
C     ------------------------------------------------------------------

!COMMON BLOCKS:

C     name       description
C     ------------------------------------------------------------------

!DESCRIPTION:
c
c     this subroutine is for calculating the weighting function of
c     atmospheric absorbing constituent profile (h2o, o3, etc.)
c

!ALGORITHM REFERENCES:

!KNOWN BUGS AND LIMITATIONS:

!ROUTINE HISTORY:

!END  ==================================================================

C     ================================================================

      implicit none

C-----------------------------------------------------------------------
C                             INCLUDE FILES
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C                          EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C                              ARGUMENTS
C-----------------------------------------------------------------------

C     type          name              purpose                      units
C-----------------------------------------------------------------------
c     integer       lsta              level of cloud top
c     integer       nl                level of atmosphere
c     real          cr                clear reflected radiance
c     real          dbdt              planck function derivative
c     real          emis              emissivity
c     real          tau               transmittance profile
c     real          tautr             tau(sfc)*tau(sfc)/tau(lev)
c     real          tem               temperature profile            k
c     real          tsurfair          clear air surface temperature  k
c     real          tswq              retrieved surface temperature  k

      integer       lsta
      integer       nl
      real          cr
      real          dbdt(nl)
      real          emis
      real          tau(nl)
      real          tautr(nl)
      real          tem(nl)
      real          tsurfair
      real          tswq
      real          wq(nl)

C-----------------------------------------------------------------------
C                            LOCAL VARIABLES
C-----------------------------------------------------------------------

C     type          name              purpose                      units
C-----------------------------------------------------------------------
c     integer       i                 index
c     integer       ii                index
c     integer       ip                index
c     real          phd1              wt function component
c     real          phd2              wt function component
c     real          phds1             wt function component
c     real          phds2             wt function component
c     real          sum               summation
c     real          tem1              wt function component
c     real          tem2              wt function component
c     real          tems1             wt function component
c     real          tems2             wt function component
c     real          wq1               weighting function component
c     real          wq2               weighting function component

      integer       i
      integer       ii
      integer       ip
      real          phd1
      real          phd2
      real          phds1
      real          phds2
      real          sum
      real          tem1
      real          tem2
      real          tems1
      real          tems2
      real          wq1
      real          wq2
c
      wq1=dbdt(lsta)*(tsurfair-tswq)*tau(lsta)*emis
      wq1=wq1+(-2.0*cr*tau(lsta)*tau(lsta))
c
      do 10 i=2,lsta
      wq2=0.0
      if(i.eq.lsta) go to 100
c
      phd1=dbdt(i)*tau(i)
      tem1=tem(i)
      do 20 ip=i+1,lsta
      phd2=dbdt(ip)*tau(ip)
      tem2=tem(ip)
      wq2=wq2+0.5*(phd1+phd2)*(tem2-tem1)
      phd1=phd2
      tem1=tem2
20    continue
100   wq(i)=wq1-wq2
10    continue
      return
      end






   
