c##########################################################################
	SUBROUTINE jcb_seviri(n,emis,t,tsurf,tau,taud,tauw,tauo,
     &                         maxlev,numlev,dbdtb,twf,qwf,owf,tswf)

c    T/q/O3/Ts weighting function calculation based on
c       anylitical form (Li, 1994; Li et al., 2000). For IR or MW sounder channels

!ALGORITHM REFERENCES:
c       Li Jun, 1994:    Temperature and water vapor weighting functions
c                        from radiative transfer eqaution with surface
c                        emissivity and solar reflectivity, Advances in
c                        Atmospheric Sciences, 11: 421-426.
c       Li, J., W.W.Wolf, W.P.Menzel, W.Zhang,H.-L.Huang and T.H.Achtor,
c               2000: Global soundings of the atmosphere from ATOVS
c               measurements: The algorithm and validation. Journal of
c               Applied Meteorology, 39: 1248-1268

!ROUTINE HISTORY:

C       Date            Programmer              Comments
C	25-May-07       Jun Li                  Revised for SEVIRI
C       -----------------------------------------------------------
c       29-Oct-01       Jun Li
c                       Rm.219, CIMSS/SSEC
C			University of Wisconsin-Madison
c                       1225 W. Dayton Street
c                       Madison, WI 53706
c			U.S.A.
c			Fax: (608)-262-5974
c                       Tel: (608)-2623755
c                       E-mail: jun.li@ssec.wisc.edu

        IMPLICIT none

C       --------------------------------------------------------
C       Input variables
C       --------------------------------------------------------

        INTEGER*4  n                  !  spectral band number
        REAL*4 emis                   !  Surface emissivity of that channel
        REAL*4 t(*)                   !  atmospheric temperature profile
        REAL*4 tsurf                  !  atmospheric surface skin temperature
        REAL*4 tau(*)                 !  Total transmittance
        REAL*4 taud(*)                !  Dry component transmittance
        REAL*4 tauw(*)                !  Wet component transmittance
        REAL*4 tauo(*)                !  Ozone component transmittance
        REAL*4 dbdtb                  !  deritives with respect to TBB
        INTEGER*4 maxlev              !  Maximum atmospheric levels
        INTEGER*4 numlev              !  Last level of atmospheric surface

C       --------------------------------------------------------
C       Output variables
C       --------------------------------------------------------

        REAL*4 twf(*)                 !  Temperature (K) JCB, twf(L)=dTB/dT(L)
        REAL*4 qwf(*)                 !  WV mixing ratio JCB, qwf(L)=dTB/dlnq(L)
        REAL*4 owf(*)                 !  O3 mixing ratio JCB, owf(L)=dTB/dlno(L)
        REAL*4 tswf                   !  Surface Skin T JCB,  tswf  =dTB/dTs

C       --------------------------------------------------------
C       Local variables
C       --------------------------------------------------------

        REAL*4 dbdt(200)
        REAL*4 dtau(200)
        REAL*4 tautr(200)
        REAL*4 dtautr(200)
        REAL*4 atauw(200)
        REAL*4 datauw(200)
        REAL*4 crr
        REAL*4 dbdt_metsg
        INTEGER*4 L , K

C       -----------------------------------------------------------------------------------
C       Dec.17, 1998: Tau* definition (tautr (L) = tau (numleve) * tau (numlev) / tau (L)
C       -----------------------------------------------------------------------------------
   
        DO L=1,numlev

	      IF(tau(numlev).LT.0.001) THEN
              tautr(L)=0.0
           ELSE
              tautr(L)=tau(numlev)*tau(numlev)/tau(L)
           ENDIF

        ENDDO

        crr = 0.0

C       -------------------------------------
C       Weighting Function calculations!!!
C       -------------------------------------

        DO L=1,maxlev
           dbdt(L) = dbdt_metsg(t(L),n)/dbdtb
        ENDDO

C       -------------------------------------
c       w.f. of temperature profile
C       -------------------------------------

        DO L=1,maxlev
           twf(L)=0.0
        ENDDO

C       -------------------------------------
c       clear part of twf
C       -------------------------------------

        CALL deltau(tau,dtau,numlev)

        CALL deltau(tautr,dtautr,numlev)

        DO L=1,numlev
           twf(L)=dbdt(L)*(dtau(L)-(1.0-emis)*dtautr(L))
        ENDDO

        twf(1)=0.0
        twf(2)=0.0

C       --------------------------------------------------------------
c       w.f. of Water Vapor Mixing Ratio (in terms of lnq) Profile
C       --------------------------------------------------------------

        DO L=1,maxlev
           qwf(L)=0.0
        ENDDO

        CALL getwq0(crr,qwf,tautr,t,tau,maxlev,numlev,
     $            dbdt, tsurf,t(numlev),emis)

        DO L=1,numlev

           IF(tauw(L).LT.0.001) THEN
              atauw(L)=ALOG(0.001)
           ELSE
              atauw(L)=ALOG(tauw(L))
           ENDIF

        ENDDO

        CALL deltau(atauw,datauw,numlev)

        DO L=1,numlev
           qwf(L)=qwf(L)*datauw(L)
        ENDDO

C       -------------------------------------
c       Ozone Mixing Ratio Profile
C       -------------------------------------

        DO L=1,maxlev
           owf(L)=.0
        ENDDO

        CALL getwq0(crr,owf,tautr,t,tau,maxlev,numlev,
     $             dbdt,tsurf,t(numlev),emis)

        DO L=1,numlev

           IF(tauo(L).LT.0.001) THEN
              atauw(L)=ALOG(0.001)
           ELSE
              atauw(L)=ALOG(tauo(L))
           ENDIF

        ENDDO

        CALL deltau(atauw,datauw,numlev)

        DO L=1,numlev
           owf(L)=owf(L)*datauw(L)
        ENDDO

C       -------------------------------------
c       Surface skin temperature
C       -------------------------------------

        tswf=emis*tau(numlev)*dbdt(numlev)

        RETURN
        END


