	PROGRAM	test_run

C Oct 5th, 2007
C
C * Xin Jin
C * CIMSS/SSEC
C * 1225 West Dayton Street
C * Madison, WI 53706
C * E-mail:  xjin@ssec.wisc.edu

C ------------------------------------------------------------
C * The program does the SEVIRI profile retrieval based on the 
C * regression coefficients calculated for METEROSAT-8
C ------------------------------------------------------------

	IMPLICIT  none

	INTEGER*4 nl               !  total pressure layers in the RTM model
	INTEGER*4 ncseviri         !  total SEVIRI bands
	INTEGER*4 nemis            !  number of IR emissivity
	PARAMETER (nl=101)
	PARAMETER (ncseviri=12)
	PARAMETER (nemis=8)

C --------------------------------------------------
C	Atmospheric parameters
C --------------------------------------------------
	REAL*4    p(nl)            !  pressure coordinate
	REAL*4    tf(nl)           !  forecast temperature profile (K)
	REAL*4    wf(nl)           !  forecast mixing ratio profile (g/kg)
	REAL*4    o3f(nl)          !  forecast ozone profile (ppmv)

c --------------------------------------------------
c  Variables for subrotuine (physical) input
c --------------------------------------------------
	REAL*4    tb(nl)           !  background of temperature, 101 pressure levels (K)
	REAL*4    wb(nl)           !  background of water vapor, 101 pressure levels (g/kg)
	REAL*4    tsurfb           !  background of surface skin temperature (K)
	REAL*4    b_inv(203,203)   !  inverse of the square of first guess error matrix (T/W profile + Tskin)
	REAL*4    tg(nl)           !  first guess of temperature, 101 pressure levels (K)
	REAL*4    wg(nl)           !  first guess of moisture, 101 pressure levels (g/kg)
	REAL*4    o3g(nl)          !  first guess of ozone, 101 pressure levels (ppmv)
	REAL*4    tsurfg           !  first guess of skin temperature (K)
	REAL*4    emisg(nemis)     !  first guess of surface emissivities, 8 IR spectral bands
	REAL*4    brto(ncseviri)   !  bias-adjusted BT (K), 12 spectral bands
	INTEGER*4 banduse(ncseviri)!  SEVIRI band use index
	REAL*4    radnos(ncseviri) !  MSG-1/SEVIRI NEDR
	INTEGER*4 isfc             !  1 to use surface obs, 0 not to use surface obs
	REAL*4    tsair_obs        !  surface temperature observation
	REAL*4    wsair_obs        !  surface moisture observation
	INTEGER*4 nsat             !  satellite ID (8)
	REAL*4    theta            !  local zenith angle (degree)
	REAL*4    psurf            !  surface pressure (hPa)
	INTEGER*4 lsfc             !  surface layer index
	INTEGER*4 initr            !  initialization index for physical retrieval
	INTEGER*4 if_print         !  1 print out some iteration details, 0 not to
	INTEGER*4 ib_covar         !  0 use default error covariance, 1 input from outside

c --------------------------------------------------
c  Variables for subrotuine (physical) output
c --------------------------------------------------
	REAL*4    tr(nl)           !  physical retrieval of temperauture, 101 layers (K)
	REAL*4    wr(nl)           !  physical retrieval of moisture, 101 layers (g/kg)
	REAL*4    tsurfr           !  physical retrieval of skin temperature (K)
	INTEGER*4 if_success       !  0 means failure to physical retrieval, others mean successful retrieval
	REAL*4    residual0(ncseviri+2) ! residual from the first guess (brto - brtc(x(0))
	REAL*4    residual1(ncseviri+2) ! residual from the physical retrieval (brto - brtc(x(n))
	INTEGER*4 qc_flag          ! quality flag in the physical retrieval

c --------------------------------------------------
c  	Local Variables
c --------------------------------------------------
	REAL*4    bt(8)            !  8 band brightness temperature
	REAL*4    pland            !  land/ocean flag
	REAL*4    alat             !  satellite pixel latitude
	REAL*4    alon             !  satellite pixel longitude
	REAL*4    amonth           !  numerical month (1.0 ~ 12.0)
	REAL*4    elvgeo           !  elevation angle for LZA
	REAL*4    t_ref(nl)        !  reference temperature profile for regression
	REAL*4    w_ref(nl)        !  reference water vapor mixing ratio profile for regression
	REAL*4    mlat             !  latitude of METEROSAT
	REAL*4    mlon             !  longitude of METEROSAT
	REAL*4    plstop           !  possible surface pressure for highest layer
	REAL*4    plsbot           !  possible surface pressure for lowest layer
	REAL*4    emis(nemis)      !  emisivity from outside database
	REAL*4    tcog             !  total column ozone (guess)
	REAL*4    tpwg             !  total precipitable water (guess)
	REAL*4    tpwr             !  total precipitable water (retrieval)
	REAL*4    tau(nl)          !  total atmospheric transmittance

	INTEGER*4 isem             !  1 for retrieved emissivity, 0 for emissivity database
	INTEGER*4 iprof            !  1 retrieved profiles used as FG; 0 forecast used
	INTEGER*4 initg            !  initialization index for regression retrieval
	INTEGER*4 lsurface         !  function to calculate surface layer index
	INTEGER*4 L                !  loop index

	DATA      initg/0/
	DATA      initr/0/

	DATA radnos/3*0.0,0.004601, 0.009789, 0.022604, 0.094806,
     &         0.097523, 0.124708, 0.192272, 0.417790,1*0.0/

	DATA p/0.005,.016,.038,.077,.137,.224,.345,.506,.714,
     &    .975,1.297,1.687,2.153,2.701,3.340,4.077,4.920,
     &    5.878,6.957,8.165,9.512,11.004,12.649,14.456,16.432,
     &    18.585,20.922,23.453,26.183,29.121,32.274,35.651,39.257,
     &    43.100,47.188,51.528,56.126,60.989,66.125,71.540,77.240,
     &    83.231,89.520,96.114,103.017,110.237,117.777,125.646,
     &    133.846,142.385,151.266,160.496,170.078,180.018,190.320,
     &    200.989,212.028,223.441,235.234,247.408,259.969,272.919,
     &    286.262,300.000,314.137,328.675,343.618,358.966,374.724,
     &    390.893,407.474,424.470,441.882,459.712,477.961,496.630,
     &    515.720,535.232,555.167,575.525,596.306,617.511,639.140,
     &    661.192,683.667,706.565,729.886,753.628,777.790,802.371,
     &    827.371,852.788,878.620,904.866,931.524,958.591,986.067,
     &    1013.948,1042.232,1070.917,1100.000/

C -----------------------------------------------------------
C	namelist
C -----------------------------------------------------------
	NAMELIST/input/nsat,banduse,isem,emis,amonth,
     &    if_print, isfc, tsair_obs, wsair_obs, iprof, 
     &    ib_covar, mlat, mlon, plstop, plsbot

C -----------------------------------------------------------
C	read namelist
C -----------------------------------------------------------
	OPEN(1, FILE='input.nl',STATUS='old')
	READ(1, input)
	WRITE(*, input)
	CLOSE(1)

C -----------------------------------------------------------
C	load MSG Planck-function and band-correction coefficients
C -----------------------------------------------------------
	CALL pfcometsg101(nsat,700)
	GOTO 120

700	CONTINUE
	WRITE(*,'('' Problem in I/O of msg101pfc'')')
	STOP

120	CONTINUE

C -----------------------------------------------------------
C	load sample data file for the test
C	users need to change this part for their own data
C -----------------------------------------------------------
	OPEN(55,FILE='sampledata.txt')
	READ(55,*)
	READ(55,'(8f8.3)')bt
	READ(55,*)
	READ(55,'(2f8.3)')alat,alon
	READ(55,*)
	READ(55,'(f8.3)')psurf
	READ(55,*)
	DO L=1,nl
	   READ(55,*)tf(L),wf(L),o3f(L)
	ENDDO
	CLOSE(55)

C -----------------------------------------------------------
C	calculate the surface layer index
C -----------------------------------------------------------
	lsfc=lsurface(nl,p,psurf,plstop, plsbot)

C -----------------------------------------------------------
C	prepare T/W/O profiles for regression
C	the minimun mixing ratio is set to 0.002 (g/kg)
C -----------------------------------------------------------
	DO L=1,nl
	   t_ref(L)=tf(L)
	   w_ref(L)=ALOG(wf(L))
	   if(wf(L).le.0.002) w_ref(L)=ALOG(0.002)
	ENDDO

	IF (abs(alon).GT.180.) GOTO 100   ! -180 <= alon <= +180

C -----------------------------------------------------------
C	calculate local zenith angle which should be within 80 Deg
C	to fit the requrement for regression
C -----------------------------------------------------------
	theta=90.-elvgeo(alat,alon,mlat,mlon)

	IF (theta.GE.80..OR.theta.LE.0.) GOTO 200    ! 0 < theta <= +80

C -----------------------------------------------------------
C	set brighteness temperatures for regression
C	do bias adjustment first
C -----------------------------------------------------------
	brto=0.

	DO L=1,8
	   brto(L+3)=bt(L)
	ENDDO

c--	bias adjustment
	brto(5) = 1.0209 * brto(5) - 3.6178     ! 6.2
	brto(6) = 0.9460 * brto(6) + 12.944     ! 7.2
	brto(9) = 0.8992 * brto(9) + 29.431     ! 10.8
	brto(10)= 0.8997 * brto(10)+ 29.132     ! 12.0
	brto(11)= 0.9568 * brto(11)+ 12.533     ! 13.4

C -----------------------------------------------------------
C	non-linear regression
C -----------------------------------------------------------
	CALL msgind_ges_wTQ(nsat,brto,psurf,theta,alat,
     &  amonth,pland,t_ref,w_ref,tg,wg,o3g,tsurfg,tpwg,tcog,emisg,initg)

C	------------------------------------------------------
C	prepare background parameters for physical retrieval
C	------------------------------------------------------

C--	profiles

	IF (iprof.EQ.1) THEN              ! Regression as FG
	    DO L=1,nl
	       tb(L)=tg(L)
	       wb(L)=wg(L)
	    ENDDO
	ELSE                              ! Forecast as FG
	    DO L=1,nl
	       tb(L)=tf(L)
	       wb(L)=wf(L)
	       o3g(L)=o3f(L)
	    ENDDO
	ENDIF

C--	surface skin temperature

	tsurfb = tsurfg
	print *,tsurfg
C--	Emissivity

	IF (isem.EQ.1) THEN               ! Regression as FG
	    DO L = 1, nemis
	       IF (emisg(L).GE.1.0) emisg(L)=0.98
	    ENDDO
	    emisg(1)=0.98
	    emisg(4)=0.98
	ELSE                              ! load from emissivity database
	    DO L = 1, nemis
	       emisg(L)=emis(L)
	    ENDDO
	ENDIF

C	--------------------------------------------------------
C	Physical retrieval
C	--------------------------------------------------------
	CALL seviri_phy_profile_EOF(
c-------INPUT----------------
     &  tb,             !  background of temperature, 101 pressure levels (K)
     &  wb,             !  background of water vapor, 101 pressure levels (g/kg)
     &  tsurfb,         !  background of surface skin temperature (K)
     &  b_inv,          !  inverse of background error covariance matrix
     &  tb,             !  first guess of temperature, 101 pressure levels (K)
     &  wb,             !  first guess of moisture, 101 pressure levels (g/kg)
     &  o3g,            !  first guess of ozone, 101 pressure levels (ppmv)
     &  tsurfg,         !  first guess of skin temperature (K)
     &  emisg,          !  first guess of surface emissivities, 8 IR spectral bands
     &  brto,           !  bias-adjusted BT (K), 12 channels, 0 to VIS/NIR bands
     &  banduse,        !  SEVIRI band use index
     &  radnos,         !  NeDR of SEVIRI, 12 spectral bands
     &  isfc,           !  1 to use surface obs, 0 not to use surface obs
     &  tsair_obs,      !  surface obs of temperature (K)
     &  wsair_obs,      !  surface obs of moisture (g/Kg)
     &  nsat,           !  Meteosat satellite index (8/9..)
     &  theta,          !  local zenith angle (degree)
     &  psurf,          !  surface pressure (hPa)
     &  lsfc,           !  surface pressure (hPa)
     &  initr,          !  initilization index: set initr=0 in the begaining of the main program
     &  if_print,       !  1 to print out some iteration information
     &  ib_covar,       !  0 to use defaut background covariance matrix, 1 to use input covariance matrix
c-------OUTPUT-----------------

     &  tr,             !  physical retrieval of temperauture, 101 pressure levels (K)
     &  wr,             !  physical retrieval of moisture, 101 pressure levels (g/kg)
     &  tsurfr,         !  physical retrieval of skin temperature (K)
     &  if_success,     !  0 means failure to physical retrieval, others mean successful retrieval
     &  residual0,      !  BT residual from the first guess at 12 spectral bands (K)
     &  residual1,      !  BT residual from the physical retrieval at 12 spectral bands (K)
     &  qc_flag)        !  quality flag in the physical retrieval

	CALL TPRECW(p, wr, tpwr, nl, lsfc)

	GOTO 300

100	PRINT *,'latitude should be between -180 and 180 !'
	STOP

200	PRINT *,'local zenith angle should be between 0 and 80 !'
	STOP

300	PRINT *,'The TPW = ',tpwr,'cm'

	END PROGRAM
