      SUBROUTINE TPRECW(P, W, U, NL, NP)


C-----------------------------------------------------------------------
C !F77
C
C !DESCRIPTION:
C     Compute total precipitable water given a top-down profile of
C     water vapor mixing ratios.
C
C !INPUT PARAMETERS:
C     P     Pressure profile (millibars)
C     W     Water vapor mixing ratio profile (grams/kilogram)
C     NL    Number of levels (assumed to be 101 levels in this version)
C     NP    Surface level in profile
C
C !OUTPUT PARAMETERS:
C     U     Total precipitable water vapor (centimeters)
C
C !REVISION HISTORY:
C
C !TEAM-UNIQUE HEADER:
C     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C
C !END
C-----------------------------------------------------------------------

      IMPLICIT NONE


c ... Scalar arguments
      real u
      integer nl, np

c ... Array arguments
      real p(nl), w(nl)

c ... Local scalars
      real dp, f, p1, p2, s, w1, w2
      integer i

c ... Data statements
      data f/1961.33/

c ... Integrate the profile
      w1 = w(21)
      p1 = p(21)
      s = 0.0
      do i = 22, np
          w2 = w(i)
          p2 = p(i)
          dp = abs(p2 - p1)
          s = s + (w1 + w2) * (dp / f)
          w1 = w2
          p1 = p2
      end do
      u = s

      END

