C
      FUNCTION URAND(IY)
C   TAKEN FROM FORSYTHE,MALCOLM,MOLER: "COMPUTER METHODS
C   FOR MATHEMATICAL COMPUTATIONS," PRENTICE HALL,1977,259PP.
C   THE ROUTINE CAN BE FOUND IN LAST CHAPTER, P246.
C   PROGRAMMER: LOUIS GARAND, JULY 1983
C
C   URAND IS A UNIFORM RANDOM NUMBER GENERATOR.  THE 'SEED' IY SHOULD
C   BE INITIALIZED TO AN ARBITRARY INTEGER PRIOR TO THE FIRST CALL
C   TO URAND.  THE CALLING PROGRAM SHOULD NOT ALTER THE VALUE OF IY
C   WHICH IS CHANGED INTERNALLY BY URAND ITSELF.
C   VALUES OF URAND ARE RETURNED IN THE RANGE 0. TO 1.
C
      REAL*8 HALFM,DATAN,DSQRT
      DATA M2/0/,ITWO/2/
      IF(M2.NE.0) GO TO 120
C
C * ON FIRST ENTRY, COMPUTE MACHINE INTEGER WORD LENGTH
      M=1
  110 M2=M
      M=ITWO*M2
      IF(M.GT.M2) GO TO 110
      HALFM=M2
C
C * COMPUTE MULTIPLIER AND INCREMENT FOR LINEAR CONGRUENTIAL METHOD
      IA=8*IDINT(HALFM*DATAN(1.D0)/8.D0) + 5
      IC=2*IDINT(HALFM*(0.5D0-DSQRT(3.D0)/6.D0)) +1
      MIC=(M2-IC)+M2
C * OBTAIN SCALE FACTOR FOR CONVERTING TO FLOATING POINT
      S=0.5/HALFM
C
C * COMPUTE NEXT RANDOM NUMBER
  120 IY=IY*IA
C * FOR COMPUTERS NOT ALLOWING INTEGER OVERFLOW ON ADDITION:
      IF(IY.GT.MIC) IY=(IY-M2)-M2
      IY=IY+IC
C * FOR COMPUTERS WITH WORD LENGTH FOR ADDITION GREATER THAN FOR
C    MULTIPLICATION:
      IF(IY/2.GT.M2) IY=(IY-M2)-M2
C * FOR COMPUTERS WHERE INTEGER OVERFLOW AFFECTS THE SIGN BIT:
      IF(IY.LT.0) IY=(IY+M2)+M2
      URAND=FLOAT(IY)*S
      RETURN
      END
